package com.example.demo.movies.api;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.api.PageDto;
import com.example.demo.core.api.PageDtoMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.types.service.TypeService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/movie")
public class MovieController {
    private final MovieService movieService;
    private final TypeService typeService;
    private final AgeRatingService ageRatingService;
    private final ModelMapper modelMapper;

    public MovieController(MovieService movieService, TypeService typeService, AgeRatingService ageRatingService,
            ModelMapper modelMapper) {
        this.movieService = movieService;
        this.typeService = typeService;
        this.ageRatingService = ageRatingService;
        this.modelMapper = modelMapper;
    }

    private MovieDto toDto(MovieEntity entity) {
        return modelMapper.map(entity, MovieDto.class);
    }

    private MovieEntity toEntity(MovieDto dto) {
        final MovieEntity entity = modelMapper.map(dto, MovieEntity.class);

        entity.setType(typeService.get(dto.getTypeId()));
        entity.setAgeRating(ageRatingService.get(dto.getAgeRatingId()));

        return entity;
    }

    @GetMapping
    public PageDto<MovieDto> getAll(
            @RequestParam(name = "typeId", defaultValue = "0") Long typeId,
            @RequestParam(name = "ageRatingId", defaultValue = "0") Long ageRatingId,
            @RequestParam(name = "page", defaultValue = "0") int page,
            @RequestParam(name = "size", defaultValue = Constants.DEFAULT_PAGE_SIZE) int size) {
        return PageDtoMapper.toDto(movieService.getAll(typeId, ageRatingId, page, size), this::toDto);
    }

    @GetMapping("/{id}")
    public MovieDto get(@PathVariable(name = "id") Long id) {
        return toDto(movieService.get(id));
    }

    @PostMapping
    public MovieDto create(@RequestBody @Valid MovieDto dto) {
        return toDto(movieService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public MovieDto update(@PathVariable(name = "id") Long id, @RequestBody @Valid MovieDto dto) {
        return toDto(movieService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public MovieDto delete(@PathVariable(name = "id") Long id) {
        return toDto(movieService.delete(id));
    }

    @GetMapping("/{id}/rating")
    public Double getAverageRating(@PathVariable(name = "id") Long id) {
        return movieService.getAverageRating(id);
    }
}
