package com.example.demo.users.model;

import java.util.HashSet;
import java.util.Collections;
import java.util.Set;
import java.util.Objects;
import java.util.Optional;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.usersubscription.model.UserSubscriptionEntity;

import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinTable;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.OneToMany;
import jakarta.persistence.OrderBy;
import jakarta.persistence.Table;

@Entity
@Table(name = "users")
public class UserEntity extends BaseEntity {
    @Column(nullable = false, unique = true, length = 30)
    private String email;

    @Column(nullable = false, length = 30)
    private String password;

    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL, orphanRemoval = true)
    @OrderBy("id ASC")
    private Set<UserSubscriptionEntity> userSubscriptions = new HashSet<>();

    @ManyToMany()
    @JoinTable(name = "user_movie", joinColumns = @JoinColumn(name = "userId"), inverseJoinColumns = @JoinColumn(name = "movieId"))
    @OrderBy("id ASC")
    private Set<MovieEntity> watchLater = new HashSet<>();

    public UserEntity() {
    }

    public UserEntity(String email, String password) {
        this.email = email;
        this.password = password;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public Set<UserSubscriptionEntity> getUserSubscriptions() {
        return userSubscriptions;
    }

    public void addSubscription(UserSubscriptionEntity userSubscription) {
        if (userSubscription.getUser() != this) {
            userSubscription.setUser(this);
        }
        userSubscriptions.add(userSubscription);
    }

    public void deleteSubscription(UserSubscriptionEntity userSubscription) {
        if (userSubscription.getUser() != this) {
            return;
        }
        userSubscriptions.remove(userSubscription);
    }

    public Set<MovieEntity> getWatchLater() {
        return watchLater;
    }

    public void setWatchLater(Set<MovieEntity> watchLater) {
        Set<MovieEntity> newWatchLater = new HashSet<>();
        newWatchLater.addAll(watchLater);
        this.watchLater.clear();
        this.watchLater.addAll(Optional.ofNullable(newWatchLater).orElse(Collections.emptySet()));
    }

    public void addMovieToWatchLater(MovieEntity movie) {
        watchLater.add(movie);
    }

    public void deleteMovieFromWatchLater(MovieEntity movie) {
        watchLater.remove(movie);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, email, password, userSubscriptions, watchLater);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final UserEntity other = (UserEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getEmail(), email)
                && Objects.equals(other.getPassword(), password);
    }
}
