package com.example.demo.usersubscription.model;

import java.util.Date;
import java.util.Objects;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.subscriptions.model.SubscriptionEntity;
import com.example.demo.users.model.UserEntity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;

@Entity
@Table(name = "users_subscriptions")
public class UserSubscriptionEntity extends BaseEntity {
    @ManyToOne
    @JoinColumn(name = "user_id", nullable = false)
    private UserEntity user;

    @ManyToOne
    @JoinColumn(name = "subscription_id", nullable = false)
    private SubscriptionEntity subscription;

    @Column(nullable = false)
    private Date activationDate;

    @Column(nullable = false)
    private Date expirationDate;

    public UserSubscriptionEntity() {
    }

    public UserSubscriptionEntity(UserEntity user, SubscriptionEntity subscription, Date activationDate,
            Date expirationDate) {
        this.user = user;
        this.subscription = subscription;
        this.activationDate = activationDate;
        this.expirationDate = expirationDate;
    }

    public UserEntity getUser() {
        return user;
    }

    public void setUser(UserEntity user) {
        this.user = user;
        
        if (!user.getUserSubscriptions().contains(this)) {
            user.getUserSubscriptions().add(this);
        }
    }

    public SubscriptionEntity getSubscription() {
        return subscription;
    }

    public void setSubscription(SubscriptionEntity subscription) {
        this.subscription = subscription;
    }

    public Date getExpirationDate() {
        return expirationDate;
    }

    public void setExpirationDate(Date expirationDate) {
        this.expirationDate = expirationDate;
    }

    public Date getActivationDate() {
        return activationDate;
    }

    public void setActivationDate(Date activationDate) {
        this.activationDate = activationDate;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, user.getId(), subscription.getId(), activationDate, expirationDate);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        UserSubscriptionEntity other = (UserSubscriptionEntity) obj;
        return Objects.equals(id, other.id)
                && Objects.equals(user.getId(), other.user.getId())
                && Objects.equals(subscription.getId(), other.subscription.getId())
                && Objects.equals(activationDate, other.activationDate)
                && Objects.equals(expirationDate, other.expirationDate);
    }
}
