package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.error.AlreadyExistsException;
import com.example.demo.core.error.NotFoundException;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class AgeRatingServiceTests {
	@Autowired
	private AgeRatingService ageRatingService;

	private AgeRatingEntity ageRating;

	@BeforeEach
	void createData() {
		removeData();

		ageRating = ageRatingService.create(new AgeRatingEntity("12+"));
		ageRatingService.create(new AgeRatingEntity("16+"));
		ageRatingService.create(new AgeRatingEntity("18+"));
	}

	@AfterEach
	void removeData() {
		ageRatingService.getAll().forEach(item -> ageRatingService.delete(item.getId()));
	}

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> ageRatingService.get(0L));
	}

	@Test
	void createTest() {
		Assertions.assertEquals(3, ageRatingService.getAll().size());
		Assertions.assertEquals(ageRating, ageRatingService.get(ageRating.getId()));
	}

	@Test
	void createNotUniqueTest() {
		final AgeRatingEntity nonUniqueAgeRating = new AgeRatingEntity("16+");
		Assertions.assertThrows(AlreadyExistsException.class, () -> ageRatingService.create(nonUniqueAgeRating));
	}

	@Test
	void createNullableTest() {
		final AgeRatingEntity nullableType = new AgeRatingEntity(null);
		Assertions.assertThrows(DataIntegrityViolationException.class, () -> ageRatingService.create(nullableType));
	}

	@Test
	void updateTest() {
		final String test = "TEST";
		final String oldName = ageRating.getName();
		final AgeRatingEntity newEntity = ageRatingService.update(ageRating.getId(), new AgeRatingEntity(test));

		Assertions.assertEquals(3, ageRatingService.getAll().size());
		Assertions.assertEquals(newEntity, ageRatingService.get(ageRating.getId()));
		Assertions.assertEquals(test, newEntity.getName());
		Assertions.assertNotEquals(oldName, newEntity.getName());
	}

	@Test
	void deleteTest() {
		ageRatingService.delete(ageRating.getId());
		Assertions.assertEquals(2, ageRatingService.getAll().size());

		final AgeRatingEntity newEntity = ageRatingService.create(new AgeRatingEntity("12+"));
		Assertions.assertEquals(3, ageRatingService.getAll().size());
		Assertions.assertNotEquals(ageRating.getId(), newEntity.getId());
	}
}
