package com.example.demo;

import java.time.Period;
import java.util.Date;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.subscriptions.model.SubscriptionEntity;
import com.example.demo.subscriptions.service.SubscriptionService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.usersubscription.model.UserSubscriptionEntity;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class UserServiceTests {
	@Autowired
	private TypeService typeService;
	@Autowired
	private AgeRatingService ageRatingService;
	@Autowired
	private MovieService movieService;
	@Autowired
	private UserService userService;
	@Autowired
	private SubscriptionService subscriptionService;

	private UserEntity last;
	private TypeEntity type1;
	private AgeRatingEntity ageRating1;
	private MovieEntity movie1;
	private MovieEntity movie2;
	private SubscriptionEntity subscription1;
	private SubscriptionEntity subscription2;

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> userService.get(0L));
	}

	@BeforeEach
	void createData() {
		removeData();

		userService.create(new UserEntity("myemail@mail.com", "123"));
		userService.create(new UserEntity("secondemail@mail.com", "321"));
		last = userService.create(new UserEntity("thirdemail@mail.com", "3210"));

		type1 = typeService.create(new TypeEntity("Фильм"));
		ageRating1 = ageRatingService.create(new AgeRatingEntity("18+"));
		movie1 = movieService.create(new MovieEntity("Бойцовский клуб", type1, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сотрудник страховой компании страдает хронической бессонницей и отчаянно пытается вырваться из мучительно скучной жизни...",
				1999, "США", "Интриги. Хаос. Мыло", "Дэвид Финчер", ageRating1,
				"https://www.youtube.com/embed/dfeUzm6KF4g?si=U2D-2WNsJzkcxlBX"));
		movie2 = movieService.create(new MovieEntity("Американский психопат", type1, true,
				"src/assets/main-page/posters/AmericanPsycho.jpg",
				"Днем он ничем не отличается от окружающих, и в толпе вы не обратите на него внимания...",
				2000, "США, Канада", "Респектабельная внешность дьявола", "Мэри Хэррон", ageRating1,
				"https://www.youtube.com/embed/PpUWjff_OcM?si=e7tSm4V6qYkz406B"));

		subscription1 = subscriptionService.create(new SubscriptionEntity("Подписка 1", Period.of(0, 0, 7), 199.99));
		subscription2 = subscriptionService.create(new SubscriptionEntity("Подписка 2", Period.of(0, 1, 0), 299.99));
	}

	@AfterEach
	void removeData() {
		userService.deleteAll();
		movieService.deleteAll();
		typeService.deleteAll();
		ageRatingService.deleteAll();
		subscriptionService.deleteAll();
	}

	@Test
	void createTest() {
		Assertions.assertEquals(3, userService.getAll().size());
		Assertions.assertEquals(last, userService.get(last.getId()));
	}

	@Test
	void updateTest() {
		final String test = "TEST";
		final UserEntity entity = userService.get(last.getId());
		final String oldEmail = entity.getEmail();
		final UserEntity newEntity = userService.update(entity.getId(),
				new UserEntity(test, entity.getPassword()));

		Assertions.assertEquals(3, userService.getAll().size());
		Assertions.assertEquals(newEntity, userService.get(newEntity.getId()));
		Assertions.assertEquals(test, newEntity.getEmail());
		Assertions.assertNotEquals(oldEmail, newEntity.getEmail());
	}

	@Test
	void deleteTest() {
		userService.delete(last.getId());
		Assertions.assertEquals(2, userService.getAll().size());

		final var newEntity = userService.create(new UserEntity("thirdemail@mail.com", "3210"));
		Assertions.assertEquals(3, userService.getAll().size());
		Assertions.assertNotEquals(last.getId(), newEntity.getId());
	}

	@Test
	void watchLaterTest() {
		userService.addMovieToWatchLater(last.getId(), movie1);
		userService.addMovieToWatchLater(last.getId(), movie1);
		Assertions.assertEquals(1, userService.getWatchLater(last.getId()).size());
		userService.addMovieToWatchLater(last.getId(), movie2);
		Assertions.assertEquals(2, userService.getWatchLater(last.getId()).size());

		userService.deleteMovieFromWatchLater(last.getId(), movie1);
		Assertions.assertEquals(1, userService.getWatchLater(last.getId()).size());
	}

	@Test
	void subscriptionsTest() {
		Assertions.assertNull(userService.getActiveSubscription(last.getId()));

		userService.activateSubscription(last.getId(), subscription1.getId());
		userService.activateSubscription(last.getId(), subscription1.getId());
		Assertions.assertEquals(1, userService.getUserSubscriptions(last.getId()).size());

		UserSubscriptionEntity activeSubscription = userService.getActiveSubscription(last.getId());
		Assertions.assertNotNull(activeSubscription);
		Assertions.assertTrue(activeSubscription.getExpirationDate().after(new Date()));

		userService.deactivateSubscription(last.getId());
		Assertions.assertNull(userService.getActiveSubscription(last.getId()));

		userService.activateSubscription(last.getId(), subscription2.getId());
		Assertions.assertNotNull(userService.getActiveSubscription(last.getId()));
		Assertions.assertEquals(2, userService.getUserSubscriptions(last.getId()).size());
	}
}
